from .download_specs import create_data_dir, get_supported_courses, SPEC_URLS
from .filter_specs import filter_assignments, find_all_specs
from .load import load_specs
from .util import check_architecture, check_spec_dependencies, get_filenames
