from .base import format_collected_data
from .html import format_assignment_html as html
from .markdown import format_assignment_markdown as markdown
from .truncate import truncate
from .yaml import format_assignment_yaml as yaml
